;**********************************************************
;*  fc002a.asm - Frequency Counter
;*
;*  Copyright 2005, 2009 Douglas Rice, 
;*
;*	www.dougrice.plus.com, 
;*	doug.h.rice@btinternet.com
;*  http://www.dougrice.plus.com/hp/freqCounter/freqCounter.html
;*
;**********************************************************
;*

;*  This program configures a PIC 12F675 to measure the frequency of a signal.
;*
;*	The frequency is measured over a 10 second period, or if GP1 is pulled low 0.5 seconds.
;*  Apply the signal to be measured to GP5 on Pin 2.
;*
;*	The frequency to be measured clocks the TMR1 exactly 10 second timed by cycle counting.
;*	
;*  NOTE: If erasing and reprogramming the 12F675, remember to record and reuse the OSCAL value at 0x3FF.
;*
;*
;*	Quasar sell a kit that measure temperature with a 12C508. You can replace the IC with a 12F675 programmmed using this code.
;*  http://www.quasarelectronics.com/3145-ds18s20-computer-serial-temperature-data-logger.htm
;*
;*  The serial output is on GP2 to suit Quasar kit. Unfortunately this prevents the use of GP2/AN2/T0CKI/INT/COUT
;*
;*  However it is possible to use TMR1. on GP5/T1CKI/OSC1/CLKIN on Pin2. This is brought out to a screw terminal on the left of the module 
;*  and pulled high by a 2K resistor.
;* 
;*  I have connected a Hamlin ( www.hamlin.com ) 55100 3M02A 0922 Hall effect switch and I see a count when wave a fridge magnet over the back. 
;*  The Sensors have red,blue and black wire. 
;* 	
;*	Left connector of quasar logger module
;*   ( DS4 Vdd )   	Connect red wire from Hamlin to 55100 3M02A 0922 Hall effect switch 
;*   ( DS4 signal )	Connect blue wire from Hamlin to 55100 3M02A 0922 Hall effect switch 
;*   ( DS4 Gnd )	Connect black wire from Hamlin to 55100 3M02A 0922 Hall effect switch 
;*
;*   ( DS3 Vdd )	N/C not connected
;*   ( DS3 signal ) N/C not connected
;*   ( DS3 Gnd )	N/C not connected
;*
;*   ( DS2 Vdd )
;*   ( DS2 signal ) Connect to DS2 Gnd to read every 0.5 seconds in stead of 10 seconds.
;*   ( DS2 Gnd )
;*	
;*   ( DS1 Vdd )	N/C not connected
;*   ( DS1 signal ) N/C not connected
;*   ( DS1 Gnd )	N/C not connected
;*
;*	Right connector
 
;*	I have not tried this counter above 115 Khz, with an oscillator.
;*
;*	Connect GP2 direct to the serial input of your PC set to 1200 baud.
;*
;*
;*	It outputs f("0,0A,00116074,0"); This is csv and is a format that can be read using Excel or JavaScript ( See: http://www.dougrice.plus.com/rt004/sr3/Logger.html ) 
;*		and the 2nd parameter is the frequency in decimal hertz. 
;* 		where the 3rd Paramter in the state of the GPIO in HEX, so it can be used to read the inputs, 
;*
;* Output at startup and waving a magnet across the hamlin 55100 3M02A Hall effect sensor. 
;* fc002a
;* f("0,00000000,3B,0");
;* f("0,00000020,3B,0");
;* f("0,00000010,3B,0");
;*
;*  Copyright 2005, 2009 Douglas Rice
;*
;*
;**********************************************************

;--------------------------------------------------------------------------
; Sec 1.0	Configuration
;-------------------------------------------------------------------------

	list p=12f675

	; Include file, change directory if needed
	include "p12f675.inc"
	__idlocs  0x1234
	


; Using internal 4Mhz Osc	
	__config  _PWRTE_ON & _WDT_OFF & _BODEN_OFF & _INTRC_OSC_NOCLKOUT & _MCLRE_OFF & 0x31FF

; Using external 4Mhz Osc	
;	__config  _PWRTE_ON & _WDT_OFF & _BODEN_OFF & _XT_OSC & _MCLRE_OFF & 0x31FF


; use GP0 as shown in the diagram on web site.
;TxOnBit  equ 0

; use GP2 to use the Quasar Temperature logger kit, unmodified.
; http://www.quasarelectronics.com/3145-ds18s20-computer-serial-temperature-data-logger.htm
TxOnBit  equ 2

; Selec the correct one for the RS232 speed

;SR_DELAY_CONST  EQU	0x03  ; delay for 32K
;SR_DELAY_CONST  EQU	0xCD  ; delay for 1200 baud ; 833us
;SR_DELAY_CONST  EQU	0x64  ; delay for 2400 baud ; 416us got 417us
SR_DELAY_CONST  EQU	0x30  ; delay for 4800 baud ; 416us got 417us



;--------------------------------------------------------------------------
; Sec 2.0	Variables
;-------------------------------------------------------------------------

	cblock 	0x20
	; variables used for context saving 
	w_temp
	status_temp

	; Serial Out routine vars
	SRtxTemp
	SRtxCnt
	SRout
	SRdel

	endc
	
	cblock		; variables used in BCD.INC
	BCDip0	
	BCDip1	
	BCDip2	
	BCDip3
	
	BCDop0                    ; RAM Assignments
	BCDop1      
	BCDop2      
	BCDop3      
	BCDop4      

	BCDcount  
	BCDtemp   
	endc

;--------------------------------------------------------------------------
; Sec 3.0	Macros
;-------------------------------------------------------------------------

TEST_STRADDLE	MACRO	START
	if high( $ ) != high( START ) 
	    Error "Table straddles Page Boundary "
	endif
	endm


;--------------------------------------------------------------------------
; Sec 4.0	Code
;-------------------------------------------------------------------------
	; Start at the reset vector
	org	0x000
	goto	Start

	org	0x010
	
Start
;	banksel 0x3FF
;	CALL 	0x3FF
	MOVWF	OSCCAL
	banksel	TRISIO 

	BSF		STATUS, RP0
	movlw	0x1F
	MOVWF	TRISIO		; Set output high to start Measurement
	BCF		STATUS, RP0

	movlw	0x0D
	call	SRtxChar

	movlw	0x0A
	call	SRtxChar

	movlw	'f'
	call	SRtxChar
	movlw	'c'
	call	SRtxChar
	movlw	'0'

	call	SRtxChar
	movlw	'0'

	call	SRtxChar
	movlw	'2'

	call	SRtxChar
	movlw	'a'

	call	SRtxChar
	movlw	';'

MainLoop
	; output f(0,freq,0);
	call	FRfreqMeterSpeedSet
	
	; do measurement.

	; The software reads the TMR1L when it can and works out how many ticks since last tick.
	; The counter keeps counting input pulses.
	; It accumulates the ticks over an exact time period, measured by the number of instruction cycles.
	; a 4Mhz PIC does 1Million instructions per second.
	;
	; Each measurement decrement the cycles count until all 1 Million instructions.
	;
	
	call	FRfreqMeterStart
;	call	FRfreqMeterConvert
	
	;
	; Start Output
	;
	; This output is CSV formatted and can be read by JavaScript.
	;
	movlw	0x0D
	call	SRtxChar

	movlw	0x0A
	call	SRtxChar

	movlw	'f'
	call	SRtxChar
	movlw	'('
	call	SRtxChar
	movlw	'"'
	call	SRtxChar
	movlw	'0'
	call	SRtxChar
	movlw	','
	call	SRtxChar

	; go off and display the count measured over 10 or 2 seconds.
	call	FRdispCount

	movlw	','
	call	SRtxChar

	; display state of inputs.
	banksel	GPIO
	swapf	GPIO,W		;
	call 	SRwrtHexNibble
	movf	GPIO,W		;
	call 	SRwrtHexNibble


	movlw	','
	call	SRtxChar
	movlw	'0'
	call	SRtxChar
	movlw	'"'
	call	SRtxChar

	movlw	')'
	call	SRtxChar

	movlw	';'
	call	SRtxChar

	goto	MainLoop		;Do it again


;--------------------------------------------------------------------------
; Sec 4.1	Serial Output Code.
;-------------------------------------------------------------------------

SRwrtHexNibble	; currently only displays 0..9, A..F
	ANDLW	0x0F
	ADDLW	0x06	; is it A..F, if so trigger a digit overflow
	SKPNDC
	ADDLW	7; subtract 10, then add 'A'-'0'
	ADDLW	0x30-6	; Subtract extra 6 added to cause DC
	MOVWF	SRout
	GOTO	SRtxChar



SRtxChar
	;
	; Send Character to the RS232 pin.
	;
	; Output Start Bit
	;
	; Output LSB first

	movwf	SRtxTemp
	comf	SRtxTemp,f
	
	BCF		GPIO,TxOnBit
	MOVLW	~( 1 << TxOnBit ) 
	
;	TRIS	GPIO
	BSF		STATUS, RP0
	MOVWF	TRISIO		; Set output high to start Measurement
	BCF		STATUS, RP0

	; 1200 baud output
	; move the char into SRtxtemp, it is destroyed.
	;Start Bit
	BSF		GPIO,TxOnBit
	GOTO	$+1
	call	delay	
	movlw	4
	movwf	SRtxCnt
	GOTO 	$+1
SRtxCharLp1

	; This outputs two bits at a time 
	; this takes 6.5 cycles each ( 1200  when I used 32.768kHz Oscillator
	
	; After start bit, which is 1, then test each bit
	; I need to set PA:0 to same as LSB SRtxTemp
	; this compares PA:0 with SRtxTemp so see if it needs toggling
	; The xorwf PORTA,f causes the bit to be toggled if required.

	; Toggle bit if next bit is different 
	RRF		SRtxTemp,f
	movfw	STATUS
	andlw	1 << C		; mask off bit
	addlw   ( 1 << TxOnBit) -1	; shift bit by using a ripple carry
		
	xorwf	GPIO,W
	andlw	0x1 << TxOnBit
	xorwf	GPIO,f		;t+6,19

	GOTO	$+1	
	call	delay	
	RRF		SRtxTemp,f

	movfw	STATUS
	andlw	1 << C
	addlw   ( 1 << TxOnBit) -1

	xorwf	GPIO,W
	andlw	0x1 << TxOnBit
	xorwf	GPIO,f		;t+11,24
	GOTO	$+1	
	call	delay	
	decfsz	SRtxCnt
	goto	SRtxCharLp1
	GOTO	$+1	
	GOTO	$+1	
	; output stop bit
	BCF		GPIO,TxOnBit
	call	delay	
	GOTO	$+1	
	GOTO	$+1	
	RETURN





delay	
	; one bit delay - trimed using Stopwatch and simulate.
	nop
;	movlw	0x03  ; delay for 32K
;	movlw	0xCD  ; delay for 1200 baud ; 833us
;	movlw	0x64  ; delay for 2400 baud ; 416us got 417us

;	movlw	0x30  ; delay for 4800 baud ; 416us got 417us

	movlw	SR_DELAY_CONST  ; delay for 2400 baud ; 416us got 417us


	movwf	SRdel
delay1	
	NOP
	decfsz	SRdel
	goto	delay1
	return

;-------------------------------------------------------------------------
; Sec 4.2	Frequency Counter Code
;-------------------------------------------------------------------------
	cblock
	    FRzero
		current
		lasttmr0
		;BCDip0,BCDip1,BCDip2,BCDip3
		tc0,tc1,tc2,tc3,tc4
	endc

	


FRfreqMeter1
	; we arm to display first of all
	NOP
	return


FRfreqMeterSpeedSet

	; turn off A to D converter
	banksel  ADCON0
	movlw	B'00000000'	;
	movwf	ADCON0

	banksel  ANSEL
	movlw	B'00000000'	;Fosc/8, A/D disabled
	movwf	ANSEL

	; set up Vref for Comparator. Turn it of as it is not used.
	banksel  VRCON
	movlw	0 << VREN | 0<<VRR | 5	; level = Vdd * 4 / 24 = 1.0 Volts if Vdd is 5 volts.
	movwf	VRCON

FRfreqMeterNoComparator
	; Make sure that the pin is not used by the A to D 
	; Make sure that the Comparator is disabled

	; make the GPIO all inputs
	banksel  TRISIO
	movlw	B'11111111'	;Fosc/8, A/D enabled
	movwf	TRISIO

	; Configure the Comparator Off (Lowest power)
	banksel  CMCON
	movlw	B'00010111'	;
	movwf	CMCON

	; make the GPIO all inputs
	banksel  TRISIO
	movlw	B'11111111'	;Fosc/8, A/D enabled
	movwf	TRISIO

	; Change Prescaler and connect TMR0 ip to external
	CLRWDT
	; Configure Tmr 0
	
	; set TMR0 to increment off internal input for test purposes - CLK off internal Clock
	; MOVLW	0x0 + 0x00  + 0 	; T0CS + For Tmr0 0=/2, 1=/4, 2=/8,3=/16

	; Set up CLK off external input with prescale set to 1/2.
	MOVLW	0x0 + 0x20  + 0 	; T0CS + For Tmr0 0=/2, 1=/4, 2=/8,3=/16

	banksel	OPTION_REG
	MOVWF	OPTION_REG
	
	; set up Timer1 to count signal on Pin 2 
	movlw	0 << TMR1GE | 0 << T1CKPS1 | 0 << T1CKPS0 | 0 << T1OSCEN | 0 << NOT_T1SYNC | 1 << TMR1CS | 1 <<  TMR1ON

	banksel	T1CON
	movwf	T1CON

	banksel lasttmr0
	clrf	lasttmr0
	
	
	banksel BCDip0
	clrf	BCDip0
	clrf	BCDip1
	clrf	BCDip2
	clrf	BCDip3
	; call delay to allow comparator to settle down
	call	delay
	call	delay


	; test GP1 to see if fast mode set 
	banksel	GPIO
	btfsc	GPIO,1
	goto	FRslow


FRfast
    ; measure over 0.5 secnds if GP1 is pulled low.
	; 500,000 = 7 A1 20
	; using prescale of div 2
	; These seem to work, 
	; but still underead a litte bit
	movlw	0x21
	movwf	tc0
	movlw	0xA2
	movwf	tc1
	movlw	0x8
	movwf	tc2
	return
	
	
FRslow
    ; measure over 0.5 secnds if GP1 is high.

	; 10,000,000 = 98 96 80
	; using prescale of div 2
	; These seem to work, 
	; but still underead a litte bit
	movlw	0x81
	movwf	tc0
	movlw	0x97
	movwf	tc1
	movlw	0x99
	movwf	tc2
	return


FRincCnt	
	; get current count, This is volatile so sample it and store it away.
	; negate last 
	movfw	current
	movwf	lasttmr0
;	movfw	TMR0
	movfw	TMR1L
	
	; w = Current - lasttmr0
	movwf	current
	subwf	lasttmr0,w
	
	sublw	0

	addwf	BCDip0,f
	movfw	STATUS
	andlw	1
	addwf	BCDip1,f
	
	movfw	STATUS
	andlw	1
	addwf	BCDip2,f

	movfw	STATUS
	andlw	1
	addwf	BCDip3,f
		
	retlw   -.29
	
	; we don't need to worry about the carry.
	;
	; This is complicated as skips take one or two instructions.
	; and we are counting cycles to get exactly 2 or 10 seconds.
	;sublw	0
	;addwf	BCDip0,f		; t
	;skpc			; t+0 
	;retlw	-.24		; t+1   these values are used to adjust the main timing loop.
	;incfsz	BCDip1,f		; t+2
	;retlw	-.26		; t+3
	;incfsz	BCDip2,f		; t+4
	;retlw	-.28		; t+5
	;incfsz	BCDip3,f		; t+6
	;retlw	-.30		; t+7


;-- WARNING ------------------------------------------------------
    ; start measuring frequency HERE as we sample the count
    ; adding instructions here affects the readings
;	movfw	TMR0

FRfreqMeterStart
    clrf	FRzero
	movfw	TMR1L
	movwf	current
    nop

FRfreqMeter0
	;	12 cycles
	call	FRincCnt

FreqDelay
	; w contains a fiddle factor to adjust this delay
	; get current count 
	
	; we don't need to worry about the carry.
	addwf	tc0,f		;t
				;t= 0
	skpnc
	goto	fd1		;t+1 these values are used to adjust the main timing loop.
			;
	decfsz	tc1,f		;t+2
	goto	fd2		;t+3
	decfsz	tc2,f		;t+4
	goto	fd3		;t+5
	goto 	fd4
	
fd1	
	; replace NOP instructions with a test to see if buttons have been pressed
	; note that theses also take 2 clock cycles

	nop			;t+5
	nop			;t+6

fd2
	nop			;t+5
	nop			;t+6

fd3	goto	FRfreqMeter0	;t+7

fd4	goto	FRdisplayResult
;fd4	return


FRdisplayResult
	; we have a reading so arm to display it.
	; make the GPIO all inputs
	banksel  TRISIO
	movlw	B'11111111'	;Fosc/8, A/D enabled
	movwf	TRISIO

	; Configure the Comparator to low power
	banksel  CMCON
	movlw	B'00000111'	;
	movwf	CMCON


	; make the GPIO all inputs
	banksel  TRISIO
	movlw	B'11111111'	;Fosc/8, A/D enabled
	movwf	TRISIO

FRfreqMeter3
	

	convert
#DEFINE	buildbcdBinDec24
;	BCF	PORTA,0

;	call	bcdBinDec24
	call    B2_BCD          ; After conversion the Decimal Number
	return
	

FRdispCount
	; now set up the display for the display.

	SWAPF	BCDop3,w
	CALL	SRwrtHexNibble

	MOVFW	BCDop3
	CALL	SRwrtHexNibble

	SWAPF	BCDop2,w
	CALL	SRwrtHexNibble

	MOVFW	BCDop2
	CALL	SRwrtHexNibble

	SWAPF	BCDop1,w
	CALL	SRwrtHexNibble

	MOVFW	BCDop1
	CALL	SRwrtHexNibble

	SWAPF	BCDop0,w
	CALL	SRwrtHexNibble

	MOVFW	BCDop0
	CALL	SRwrtHexNibble

	; we have displayed it so prevent another display
	return


;--------------------------------------------------------------------------
; Sec 4.3	Binary to BCD -Procedure
;-------------------------------------------------------------------------
	include	"bcd.inc"

	end


